@@ -19,10 +19,25 @@ void R_glp_solve (int *lp_direction, int *lp_number_of_constraints,
 		  int *lp_bounds_type, double *lp_bounds_lower,
 		  double *lp_bounds_upper,
 		  double *lp_optimum,
+		  int *lp_col_stat,
 		  double *lp_objective_vars_values,
-		  int *lp_verbosity, int *lp_status) {
+		  double *lp_objective_dual_values,
+		  int *lp_row_stat,
+		  double *lp_row_prim_aux,
+		  double *lp_row_dual_aux,
+		  int *lp_verbosity,
+		  int *lp_presolve,
+		  int *lp_time_limit,
+		  int *lp_status,
+		  int *write_fmt,
+		  char **fname) {
 
+  // GLPK problem object
   glp_prob *lp;
+  // GLPK simplex control object
+  glp_smcp control_sm;
+  // GLPK mixed integer control object
+  glp_iocp control_io;
   int i, kl, ku;
   jmp_buf env;
 
@@ -35,7 +50,7 @@ void R_glp_solve (int *lp_direction, int *lp_number_of_constraints,
 
     // create problem object 
     lp = glp_create_prob();
-    
+
     // Turn on/off Terminal Output
     if(*lp_verbosity==1)
       glp_term_out(GLP_ON);
@@ -52,26 +67,28 @@ void R_glp_solve (int *lp_direction, int *lp_number_of_constraints,
     //if(lp_integer)
     //lpx_set_class(lp, LPX_MIP);
     // add rows to the problem object
-    glp_add_rows(lp, *lp_number_of_constraints);
-    for(i = 0; i < *lp_number_of_constraints; i++)
-      switch(lp_direction_of_constraints[i]){
-      case 1: 
-	glp_set_row_bnds(lp, i+1, GLP_UP, 0.0, lp_right_hand_side[i]);
-	break;
-      case 2: 
-	glp_set_row_bnds(lp, i+1, GLP_UP, 0.0, lp_right_hand_side[i]);
-	break;
-      case 3: 
-	glp_set_row_bnds(lp, i+1, GLP_LO, lp_right_hand_side[i], 0.0);
-	break;
-      case 4: 
-	glp_set_row_bnds(lp, i+1, GLP_LO, lp_right_hand_side[i], 0.0);
-	break;
-      case 5: 
-	glp_set_row_bnds(lp, i+1, GLP_FX, lp_right_hand_side[i],
-			 lp_right_hand_side[i]);
-	break;
-      }
+    if( *lp_number_of_constraints > 0 ){
+      glp_add_rows(lp, *lp_number_of_constraints);
+      for(i = 0; i < *lp_number_of_constraints; i++)
+	switch(lp_direction_of_constraints[i]){
+	case 1: 
+	  glp_set_row_bnds(lp, i+1, GLP_UP, 0.0, lp_right_hand_side[i]);
+	  break;
+	case 2: 
+	  glp_set_row_bnds(lp, i+1, GLP_UP, 0.0, lp_right_hand_side[i]);
+	  break;
+	case 3: 
+	  glp_set_row_bnds(lp, i+1, GLP_LO, lp_right_hand_side[i], 0.0);
+	  break;
+	case 4: 
+	  glp_set_row_bnds(lp, i+1, GLP_LO, lp_right_hand_side[i], 0.0);
+	  break;
+	case 5: 
+	  glp_set_row_bnds(lp, i+1, GLP_FX, lp_right_hand_side[i],
+			   lp_right_hand_side[i]);
+	  break;
+	}
+    }
     
     // add columns to the problem object
     glp_add_cols(lp, *lp_number_of_objective_vars);
@@ -89,13 +106,39 @@ void R_glp_solve (int *lp_direction, int *lp_number_of_constraints,
     // IMPORTANT: as glp_load_matrix requires triplets as vectors of the
     // form: ia[1] ... ia[n], we have to pass the pointer to the adress
     // [-1] of the corresponding vector 
+    if( *lp_number_of_constraints > 0 ){
+      glp_load_matrix(lp, *lp_number_of_values_in_constraint_matrix,
+		      &lp_constraint_matrix_i[-1],
+		      &lp_constraint_matrix_j[-1], &lp_constraint_matrix_values[-1]);
+    }
+
+    // write lp to file
+    // mps_fixed := 1L, mps_free := 2L
+    if ( *write_fmt > 0 ) {
+        const char *out_name = fname[0];
+        if ( *write_fmt < 3 ) {
+            *lp_status = glp_write_mps(lp, *write_fmt, NULL, out_name);
+        } else if ( *write_fmt == 3 ) {
+            *lp_status = glp_write_lp(lp, NULL, out_name);
+        } else {
+            int future_flag = 0;
+            *lp_status = glp_write_prob(lp, future_flag, out_name);
+        }
+        glp_delete_prob(lp);
+        return;
+    }
     
-    glp_load_matrix(lp, *lp_number_of_values_in_constraint_matrix,
-		    &lp_constraint_matrix_i[-1],
-		    &lp_constraint_matrix_j[-1], &lp_constraint_matrix_values[-1]);
+    // set optimizer control parameters
+    glp_init_smcp(&control_sm);
+    if (*lp_time_limit > 0) {
+      control_sm.tm_lim = *lp_time_limit;
+    }
+    if (*lp_presolve == 1) {
+      control_sm.presolve = GLP_ON;
+    }
     
     // run simplex method to solve linear problem
-    glp_simplex(lp, NULL);
+    glp_simplex(lp, &control_sm);
     
     // retrieve status of optimization
     *lp_status = glp_get_status(lp);
@@ -103,10 +146,27 @@ void R_glp_solve (int *lp_direction, int *lp_number_of_constraints,
     *lp_optimum = glp_get_obj_val(lp);
     // retrieve values of objective vars
     for(i = 0; i < *lp_number_of_objective_vars; i++) {
+      lp_col_stat[i] = glp_get_col_stat(lp, i+1);
       lp_objective_vars_values[i] = glp_get_col_prim(lp, i+1);
+      lp_objective_dual_values[i] = glp_get_col_dual(lp, i+1);
+    }
+    // retrieve primal/dual multipliers
+    for(i = 0; i < *lp_number_of_constraints; i++) {
+      lp_row_stat[i] = glp_get_row_stat(lp, i+1);
+      lp_row_prim_aux[i] = glp_get_row_prim(lp, i+1);
+      lp_row_dual_aux[i] = glp_get_row_dual(lp, i+1);
     }
     if(*lp_is_integer) {
-      glp_intopt(lp, NULL);
+      // set optimizer control parameters
+      glp_init_iocp(&control_io);
+      if (*lp_time_limit > 0) {
+	control_io.tm_lim = *lp_time_limit;
+      }
+      if (*lp_presolve == 1) {
+	control_io.presolve = GLP_ON;
+      }
+      // optimize
+      glp_intopt(lp, &control_io);
       // retrieve status of optimization
       *lp_status = glp_mip_status(lp);
       
@@ -116,6 +176,10 @@ void R_glp_solve (int *lp_direction, int *lp_number_of_constraints,
       for(i = 0; i < *lp_number_of_objective_vars; i++){
 	lp_objective_vars_values[i] = glp_mip_col_val(lp, i+1);
       }
+      // retrieve MIP auxiliary variable values
+      for(i = 0; i < *lp_number_of_constraints; i++) {
+	lp_row_prim_aux[i] = glp_mip_row_val(lp, i+1);
+      }
     }
     // delete problem object
     glp_delete_prob(lp);
